@@ -11,6 +11,8 @@
 
 namespace ixion {
 
+general_error::general_error() : m_msg() {}
+
 general_error::general_error(const std::string& msg) :
     m_msg(msg) {}
 
@@ -21,25 +23,26 @@ const char* general_error::what() const throw()
     return m_msg.c_str();
 }
 
+void general_error::set_message(const std::string& msg)
+{
+    m_msg = msg;
+}
+
 file_not_found::file_not_found(const std::string& fpath) :
-    m_fpath(fpath)
+    general_error(fpath)
 {
+    std::ostringstream os;
+    os << "specified file not found: " << fpath;
+    set_message(os.str());
 }
 
-file_not_found::~file_not_found() throw()
-{
-}
-
-const char* file_not_found::what() const throw()
-{
-    std::ostringstream oss;
-    oss << "specified file not found: " << m_fpath;
-    return oss.str().c_str();
-}
+file_not_found::~file_not_found() throw() {}
 
 model_context_error::model_context_error(const std::string& msg, error_type type) :
     general_error(msg), m_type(type) {}
 
+model_context_error::~model_context_error() throw() {}
+
 model_context_error::error_type model_context_error::get_error_type() const
 {
     return m_type;
