@@ -38,17 +38,17 @@ struct builtin_func
 };
 
 const builtin_func builtin_funcs[] = {
-    { "MAX", func_max },
-    { "MIN", func_min },
-    { "AVERAGE", func_average },
-    { "WAIT", func_wait },
-    { "SUM", func_sum },
-    { "COUNTA", func_counta },
-    { "IF", func_if },
-    { "LEN", func_len },
-    { "CONCATENATE", func_concatenate },
-    { "NOW", func_now },
-    { "SUBTOTAL", func_subtotal },
+    { "MAX",         formula_function_t::func_max },
+    { "MIN",         formula_function_t::func_min },
+    { "AVERAGE",     formula_function_t::func_average },
+    { "WAIT",        formula_function_t::func_wait },
+    { "SUM",         formula_function_t::func_sum },
+    { "COUNTA",      formula_function_t::func_counta },
+    { "IF",          formula_function_t::func_if },
+    { "LEN",         formula_function_t::func_len },
+    { "CONCATENATE", formula_function_t::func_concatenate },
+    { "NOW",         formula_function_t::func_now },
+    { "SUBTOTAL",    formula_function_t::func_subtotal },
 };
 
 size_t builtin_func_count = sizeof(builtin_funcs) / sizeof(builtin_func);
@@ -121,7 +121,7 @@ formula_function_t formula_functions::get_function_opcode(const char* p, size_t
         if (match_func_name(builtin_funcs[i].name, p, n))
             return builtin_funcs[i].oc;
     }
-    return func_unknown;
+    return formula_function_t::func_unknown;
 }
 
 const char* formula_functions::get_function_name(formula_function_t oc)
@@ -147,40 +147,40 @@ void formula_functions::interpret(formula_function_t oc, value_stack_t& args)
 {
     switch (oc)
     {
-        case func_max:
+        case formula_function_t::func_max:
             fnc_max(args);
             break;
-        case func_average:
+        case formula_function_t::func_average:
             fnc_average(args);
             break;
-        case func_min:
+        case formula_function_t::func_min:
             fnc_min(args);
             break;
-        case func_wait:
+        case formula_function_t::func_wait:
             fnc_wait(args);
             break;
-        case func_sum:
+        case formula_function_t::func_sum:
             fnc_sum(args);
             break;
-        case func_counta:
+        case formula_function_t::func_counta:
             fnc_counta(args);
             break;
-        case func_if:
+        case formula_function_t::func_if:
             fnc_if(args);
             break;
-        case func_len:
+        case formula_function_t::func_len:
             fnc_len(args);
             break;
-        case func_concatenate:
+        case formula_function_t::func_concatenate:
             fnc_concatenate(args);
             break;
-        case func_now:
+        case formula_function_t::func_now:
             fnc_now(args);
             break;
-        case func_subtotal:
+        case formula_function_t::func_subtotal:
             fnc_subtotal(args);
             break;
-        case func_unknown:
+        case formula_function_t::func_unknown:
         default:
             throw formula_functions::invalid_arg("unknown function opcode");
     }
@@ -229,12 +229,12 @@ void formula_functions::fnc_sum(value_stack_t& args) const
     {
         switch (args.get_type())
         {
-            case sv_range_ref:
+            case stack_value_t::range_ref:
                 ret += sum_matrix_elements(args.pop_range_value());
             break;
-            case sv_single_ref:
-            case sv_string:
-            case sv_value:
+            case stack_value_t::single_ref:
+            case stack_value_t::string:
+            case stack_value_t::value:
             default:
                 ret += args.pop_value();
         }
@@ -257,18 +257,18 @@ void formula_functions::fnc_counta(value_stack_t& args) const
     {
         switch (args.get_type())
         {
-            case sv_string:
-            case sv_value:
+            case stack_value_t::string:
+            case stack_value_t::value:
                 args.pop_value();
                 ++ret;
             break;
-            case sv_range_ref:
+            case stack_value_t::range_ref:
             {
                 abs_range_t range = args.pop_range_ref();
                 ret += m_context.count_range(range, value_numeric | value_string);
             }
             break;
-            case sv_single_ref:
+            case stack_value_t::single_ref:
             {
                 abs_address_t pos = args.pop_single_ref();
                 abs_range_t range;
@@ -296,7 +296,7 @@ void formula_functions::fnc_average(value_stack_t& args) const
     {
         switch (args.get_type())
         {
-            case sv_range_ref:
+            case stack_value_t::range_ref:
             {
                 matrix mx = args.pop_range_value();
                 matrix::size_pair_type sp = mx.size();
@@ -313,9 +313,9 @@ void formula_functions::fnc_average(value_stack_t& args) const
                 }
             }
             break;
-            case sv_single_ref:
-            case sv_string:
-            case sv_value:
+            case stack_value_t::single_ref:
+            case stack_value_t::string:
+            case stack_value_t::value:
             default:
                 ret += args.pop_value();
                 ++count;
@@ -330,7 +330,6 @@ void formula_functions::fnc_if(value_stack_t& args) const
     if (args.size() != 3)
         throw formula_functions::invalid_arg("IF requires exactly 3 arguments.");
 
-    value_stack_t ret(m_context);
     value_stack_t::iterator pos = args.begin();
     bool eval = args.get_value(0) != 0.0;
     if (eval)
@@ -338,6 +337,7 @@ void formula_functions::fnc_if(value_stack_t& args) const
     else
         std::advance(pos, 2);
 
+    value_stack_t ret(m_context);
     ret.push_back(args.release(pos));
     args.swap(ret);
 }
