@@ -12,6 +12,8 @@
 #include <iostream>
 #include <sstream>
 
+#include <boost/noncopyable.hpp>
+
 #define IXION_DEBUG_LEXER 0
 
 using namespace std;
@@ -43,7 +45,6 @@ public:
     void run();
 
 private:
-    static bool is_digit(char c);
     bool is_arg_sep(char c) const;
     bool is_decimal_sep(char c) const;
     bool is_op(char c) const;
@@ -151,11 +152,6 @@ void tokenizer::run()
     }
 }
 
-bool tokenizer::is_digit(char c)
-{
-    return ('0' <= c && c <= '9');
-}
-
 bool tokenizer::is_arg_sep(char c) const
 {
     return c == m_sep_arg;
@@ -221,7 +217,7 @@ void tokenizer::numeral()
         throw formula_lexer::tokenize_error(os.str());
     }
     double val = global::to_double(p, len);
-    m_tokens.push_back(new lexer_value_token(val));
+    m_tokens.push_back(make_unique<lexer_value_token>(val));
 }
 
 void tokenizer::space()
@@ -240,7 +236,7 @@ void tokenizer::name()
         ++m_scope;
     else if (c == ']')
     {
-        m_tokens.push_back(new lexer_name_token(p, 1));
+        m_tokens.push_back(make_unique<lexer_name_token>(p, 1));
         next();
         return;
     }
@@ -264,12 +260,12 @@ void tokenizer::name()
             break;
     }
 
-    m_tokens.push_back(new lexer_name_token(p, len));
+    m_tokens.push_back(make_unique<lexer_name_token>(p, len));
 }
 
 void tokenizer::op(lexer_opcode_t oc)
 {
-    m_tokens.push_back(new lexer_token(oc));
+    m_tokens.push_back(make_unique<lexer_token>(oc));
     next();
 }
 
@@ -282,7 +278,7 @@ void tokenizer::string()
         next();
 
     if (len)
-        m_tokens.push_back(new lexer_string_token(p, len));
+        m_tokens.push_back(make_unique<lexer_string_token>(p, len));
 
     if (*mp_char == '"')
         next();
@@ -316,19 +312,7 @@ bool tokenizer::has_char() const
 
 // ============================================================================
 
-formula_lexer::tokenize_error::tokenize_error(const string& msg) :
-    m_msg(msg)
-{
-}
-
-formula_lexer::tokenize_error::~tokenize_error() throw()
-{
-}
-
-const char* formula_lexer::tokenize_error::what() const throw()
-{
-    return m_msg.c_str();
-}
+formula_lexer::tokenize_error::tokenize_error(const string& msg) : general_error(msg) {}
 
 formula_lexer::formula_lexer(const char* p, size_t n) :
     mp_first(p), m_size(n) {}
