@@ -17,6 +17,7 @@
 #include <sstream>
 #include <vector>
 #include <limits>
+#include <algorithm>
 
 #define DEBUG_NAME_RESOLVER 0
 
@@ -29,7 +30,7 @@ namespace {
 bool resolve_function(const char* p, size_t n, formula_name_type& ret)
 {
     formula_function_t func_oc = formula_functions::get_function_opcode(p, n);
-    if (func_oc != func_unknown)
+    if (func_oc != formula_function_t::func_unknown)
     {
         // This is a built-in function.
         ret.type = formula_name_type::function;
@@ -237,10 +238,53 @@ void set_cell_reference(formula_name_type& ret, const address_t& addr)
     set_address(ret.address, addr);
 }
 
-enum resolver_parse_mode {
-    resolver_parse_column,
-    resolver_parse_row
-};
+enum resolver_parse_mode { column, row };
+
+void append_sheet_name(ostringstream& os, const ixion::iface::formula_model_access& cxt, sheet_t sheet)
+{
+    string sheet_name = cxt.get_sheet_name(sheet);
+    string buffer; // used only when the sheet name contains at least one single quote.
+
+    const char* p = sheet_name.data();
+    const char* p_end = p + sheet_name.size();
+
+    bool quote = false;
+    const char* p0 = nullptr;
+
+    for (; p != p_end; ++p)
+    {
+        if (!p0)
+            p0 = p;
+
+        switch (*p)
+        {
+            case ' ':
+                quote = true;
+            break;
+            case '\'':
+                quote = true;
+                buffer += string(p0, p-p0);
+                buffer.push_back(*p);
+                buffer.push_back(*p);
+                p0 = nullptr;
+            break;
+        }
+    }
+
+    if (quote)
+        os << '\'';
+
+    if (buffer.empty())
+        os << sheet_name;
+    else
+    {
+        buffer += string(p0, p-p0);
+        os << buffer;
+    }
+
+    if (quote)
+        os << '\'';
+}
 
 void append_column_name_a1(ostringstream& os, col_t col)
 {
@@ -259,7 +303,7 @@ void append_column_name_a1(ostringstream& os, col_t col)
         col -= 1;
     }
 
-    reverse(col_name.begin(), col_name.end());
+    std::reverse(col_name.begin(), col_name.end());
     os << col_name;
 }
 
@@ -279,13 +323,7 @@ void append_address_a1(
 
     if (sheet_name_sep && cxt)
     {
-        string sheet_name = cxt->get_sheet_name(sheet);
-        bool quote = sheet_name.find_first_of(' ') != string::npos;
-        if (quote)
-            os << '\'';
-        os << cxt->get_sheet_name(sheet);
-        if (quote)
-            os << '\'';
+        append_sheet_name(os, *cxt, sheet);
         os << sheet_name_sep;
     }
 
@@ -298,6 +336,38 @@ void append_address_a1(
     os << (row + 1);
 }
 
+void append_address_r1c1(
+    ostringstream& os, const address_t& addr, const abs_address_t& pos)
+{
+    if (addr.row != row_unset)
+    {
+        os << 'R';
+        if (addr.abs_row)
+            // absolute row address.
+            os << (addr.row+1);
+        else if (addr.row)
+        {
+            // relative row address different from origin.
+            os << '[';
+            os << addr.row;
+            os << ']';
+        }
+
+    }
+    if (addr.column != column_unset)
+    {
+        os << 'C';
+        if (addr.abs_column)
+            os << (addr.column+1);
+        else if (addr.column)
+        {
+            os << '[';
+            os << addr.column;
+            os << ']';
+        }
+    }
+}
+
 void append_name_string(ostringstream& os, const iface::formula_model_access* cxt, string_id_t sid)
 {
     if (!cxt)
@@ -350,7 +420,7 @@ enum parse_address_result
 {
     invalid = 0,
     valid_address,
-    range_expected
+    range_expected /// valid address followed by a ':'.
 };
 
 #if DEBUG_NAME_RESOLVER
@@ -361,23 +431,50 @@ const char* parse_address_result_names[] = {
 };
 #endif
 
-void parse_sheet_name_quoted(const ixion::iface::formula_model_access& cxt, const char sep, const char*& p, const char* p_last, sheet_t& sheet)
+void parse_sheet_name_quoted(
+    const ixion::iface::formula_model_access& cxt, const char sep, const char*& p, const char* p_last, sheet_t& sheet)
 {
     const char* p_old = p;
     ++p; // skip the open quote.
     size_t len = 0;
+    string buffer; // used only when the name contains at least one single quote.
+    const char* p1 = p;
 
     // parse until the closing quote is reached.
     while (true)
     {
         if (*p == '\'')
         {
-            if (p == p_last || *(p+1) != sep)
+            if (p == p_last)
+                break;
+
+            if (*(p+1) == '\'')
+            {
+                // next character is a quote too.  Store the parsed string
+                // segment to the buffer and move on.
+                ++p;
+                ++len;
+                buffer += string(p1, len);
+                ++p;
+                ++len;
+                p1 = p;
+                len = 0;
+                continue;
+            }
+
+            if (*(p+1) != sep)
                 // the next char must be the separator.  Parse failed.
                 break;
 
-            const char* p1 = p_old + 1;
-            sheet = cxt.get_sheet_index(p1, len);
+            if (buffer.empty())
+                // Name contains no single quotes.
+                sheet = cxt.get_sheet_index(p1, len);
+            else
+            {
+                buffer += string(p1, len);
+                sheet = cxt.get_sheet_index(buffer.data(), buffer.size());
+            }
+
             ++p; // skip the closing quote.
             if (p != p_last)
                 ++p; // skip the separator.
@@ -426,6 +523,48 @@ void parse_sheet_name(const ixion::iface::formula_model_access& cxt, const char
     p = p_old;
 }
 
+/**
+ * If there is no number to parse, it returns 0 and the p will not
+ * increment. Otherwise, p will point to the last digit of the number when
+ * the call returns.
+ */
+template<typename T>
+T parse_number(const char*&p, const char* p_last)
+{
+    T num = 0;
+
+    bool sign = false;
+    if (*p == '+')
+        ++p;
+    else if (*p == '-')
+    {
+        ++p;
+        sign = true;
+    }
+
+    bool all_digits = false;
+    while (is_digit(*p))
+    {
+        // Parse number.
+        num *= 10;
+        num += *p - '0';
+        if (p == p_last)
+        {
+            all_digits = true;
+            break;
+        }
+        ++p;
+    }
+
+    if (!all_digits)
+        --p;
+
+    if (sign)
+        num *= -1;
+
+    return num;
+}
+
 /**
  * Parse A1-style single cell address.
  *
@@ -443,7 +582,7 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
     // the state of a value-not-set.  They are subtracted by one before
     // returning.
 
-    resolver_parse_mode mode = resolver_parse_column;
+    resolver_parse_mode mode = resolver_parse_mode::column;
 
     while (true)
     {
@@ -457,23 +596,23 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
         if ('A' <= c && c <= 'Z')
         {
             // Column digit
-            if (mode != resolver_parse_column)
+            if (mode != resolver_parse_mode::column)
                 return invalid;
 
             if (addr.column)
                 addr.column *= 26;
             addr.column += static_cast<col_t>(c - 'A' + 1);
         }
-        else if ('0' <= c && c <= '9')
+        else if (is_digit(c))
         {
-            if (mode == resolver_parse_column)
+            if (mode == resolver_parse_mode::column)
             {
                 // First digit of a row.
                 if (c == '0')
                     // Leading zeros not allowed.
                     return invalid;
 
-                mode = resolver_parse_row;
+                mode = resolver_parse_mode::row;
             }
 
             if (addr.row)
@@ -483,7 +622,7 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
         }
         else if (c == ':')
         {
-            if (mode == resolver_parse_row)
+            if (mode == resolver_parse_mode::row)
             {
                 if (!addr.row)
                     return invalid;
@@ -497,7 +636,7 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
 
                 return range_expected;
             }
-            else if (mode == resolver_parse_column)
+            else if (mode == resolver_parse_mode::column)
             {
                 // row number is not given.
                 if (addr.column)
@@ -515,12 +654,12 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
         else if (c == '$')
         {
             // Absolute position.
-            if (mode == resolver_parse_column)
+            if (mode == resolver_parse_mode::column)
             {
                 if (addr.column)
                 {
                     // Column position has been already parsed.
-                    mode = resolver_parse_row;
+                    mode = resolver_parse_mode::row;
                     addr.abs_row = true;
                 }
                 else
@@ -541,7 +680,7 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
         ++p;
     }
 
-    if (mode == resolver_parse_row)
+    if (mode == resolver_parse_mode::row)
     {
         if (!addr.row)
             return invalid;
@@ -553,7 +692,7 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
         else
             addr.column = column_unset;
     }
-    else if (mode == resolver_parse_column)
+    else if (mode == resolver_parse_mode::column)
     {
         // row number is not given.
         if (addr.column)
@@ -567,12 +706,121 @@ parse_address_result parse_address_a1(const char*& p, const char* p_last, addres
     return valid_address;
 }
 
+parse_address_result parse_address_r1c1(const char*& p, const char* p_last, address_t& addr)
+{
+    addr.row = row_unset;
+    addr.column = column_unset;
+
+    if (*p == 'R' || *p == 'r')
+    {
+        addr.row = 0;
+        addr.abs_row = false;
+
+        if (p == p_last)
+            // Just 'R'.  Not sure if this is valid or invalid, but let's call it invalid for now.
+            return parse_address_result::invalid;
+
+        ++p;
+        if (*p != 'C' && *p != 'c')
+        {
+            addr.abs_row = (*p != '[');
+            if (!addr.abs_row)
+            {
+                // Relative row address.
+                ++p;
+                if (!is_digit(*p) && *p != '-' && *p != '+')
+                    return parse_address_result::invalid;
+
+                addr.row = parse_number<row_t>(p, p_last);
+                ++p;
+                if (p == p_last)
+                    return (*p == ']') ? parse_address_result::valid_address : parse_address_result::invalid;
+                ++p;
+            }
+            else if (is_digit(*p))
+            {
+                // Absolute row address.
+                addr.row = parse_number<row_t>(p, p_last);
+                if (addr.row <= 0)
+                    // absolute address with 0 or negative value is invalid.
+                    return parse_address_result::invalid;
+
+                --addr.row; // 1-based to 0-based.
+
+                if (p == p_last && is_digit(*p))
+                    // 'R' followed by a number without 'C' is valid.
+                    return parse_address_result::valid_address;
+                ++p;
+            }
+        }
+    }
+
+    if (*p == 'C' || *p == 'c')
+    {
+        addr.column = 0;
+        addr.abs_column = false;
+
+        if (p == p_last)
+        {
+            if (addr.row == row_unset)
+                // Just 'C'.  Row must be set.
+                return parse_address_result::invalid;
+
+            if (!addr.abs_row && addr.row == 0)
+                // 'RC' is invalid as it references itself.
+                return parse_address_result::invalid;
+
+            return parse_address_result::valid_address;
+        }
+
+        ++p;
+        addr.abs_column = (*p != '[');
+        if (!addr.abs_column)
+        {
+            // Relative column address.
+            ++p;
+            if (!is_digit(*p) && *p != '-' && *p != '+')
+                return parse_address_result::invalid;
+
+            addr.column = parse_number<col_t>(p, p_last);
+            ++p;
+            if (p == p_last)
+                return (*p == ']') ? parse_address_result::valid_address : parse_address_result::invalid;
+
+            ++p;
+            if (*p == ':')
+                return (p == p_last) ? parse_address_result::invalid : parse_address_result::range_expected;
+        }
+        else if (is_digit(*p))
+        {
+            // Absolute column address.
+            addr.column = parse_number<col_t>(p, p_last);
+            if (addr.column <= 0)
+                // absolute address with 0 or negative value is invalid.
+                return parse_address_result::invalid;
+
+            --addr.column; // 1-based to 0-based.
+
+            if (p == p_last)
+                return parse_address_result::valid_address;
+
+            ++p;
+            if (*p == ':')
+                return (p == p_last) ? parse_address_result::invalid : parse_address_result::range_expected;
+        }
+    }
+    else if (*p == ':')
+        return (p == p_last) ? parse_address_result::invalid : parse_address_result::range_expected;
+
+    return parse_address_result::invalid;
+}
+
 parse_address_result parse_address_excel_a1(
     const ixion::iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
 {
     addr.row = 0;
     addr.column = 0;
-    addr.abs_sheet = false;
+    addr.abs_sheet = true; // Excel's sheet position is always absolute.
     addr.abs_row = false;
     addr.abs_column = false;
 
@@ -583,6 +831,22 @@ parse_address_result parse_address_excel_a1(
     return parse_address_a1(p, p_last, addr);
 }
 
+parse_address_result parse_address_excel_r1c1(
+    const iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
+{
+    addr.row = 0;
+    addr.column = 0;
+    addr.abs_sheet = true; // Excel's sheet position is always absolute.
+    addr.abs_row = false;
+    addr.abs_column = false;
+
+    if (cxt)
+        // Overwrite the sheet index *only when* the sheet name is parsed successfully.
+        parse_sheet_name(*cxt, '!', p, p_last, addr.sheet);
+
+    return parse_address_r1c1(p, p_last, addr);
+}
+
 parse_address_result parse_address_odff(
     const ixion::iface::formula_model_access* cxt, const char*& p, const char* p_last, address_t& addr)
 {
@@ -631,6 +895,81 @@ void to_relative_address(address_t& addr, const abs_address_t& pos)
         addr.column -= pos.column;
 }
 
+string to_string(const iface::formula_model_access* cxt, const table_t& table)
+{
+    ostringstream os;
+    append_name_string(os, cxt, table.name);
+
+    if (table.column_first == empty_string_id)
+    {
+        // Area specifier(s) only.
+        bool headers = (table.areas & table_area_headers);
+        bool data = (table.areas & table_area_data);
+        bool totals = (table.areas & table_area_totals);
+
+        short count = 0;
+        if (headers)
+            ++count;
+        if (data)
+            ++count;
+        if (totals)
+            ++count;
+
+        bool multiple = count == 2;
+        if (multiple)
+            os << '[';
+
+        append_table_areas(os, table);
+
+        if (multiple)
+            os << ']';
+    }
+    else if (table.column_last == empty_string_id)
+    {
+        // single column.
+        os << '[';
+
+        bool multiple = false;
+        if (table.areas && table.areas != table_area_data)
+        {
+            if (append_table_areas(os, table))
+            {
+                os << ',';
+                multiple = true;
+            }
+        }
+
+        if (multiple)
+            os << '[';
+
+        append_name_string(os, cxt, table.column_first);
+
+        if (multiple)
+            os << ']';
+
+        os << ']';
+    }
+    else
+    {
+        // column range.
+        os << '[';
+
+        if (table.areas && table.areas != table_area_data)
+        {
+            if (append_table_areas(os, table))
+                os << ',';
+        }
+
+        os << '[';
+        append_name_string(os, cxt, table.column_first);
+        os << "]:[";
+        append_name_string(os, cxt, table.column_last);
+        os << "]]";
+    }
+
+    return os.str();
+}
+
 } // anonymous namespace
 
 formula_name_type::formula_name_type() : type(invalid) {}
@@ -801,7 +1140,10 @@ public:
             sheet += pos.sheet;
 
         if (sheet_name && mp_cxt)
-            os << mp_cxt->get_sheet_name(sheet) << '!';
+        {
+            append_sheet_name(os, *mp_cxt, sheet);
+            os << '!';
+        }
 
         if (col != column_unset)
         {
@@ -848,77 +1190,7 @@ public:
 
     virtual string get_name(const table_t& table) const
     {
-        ostringstream os;
-        append_name_string(os, mp_cxt, table.name);
-
-        if (table.column_first == empty_string_id)
-        {
-            // Area specifier(s) only.
-            bool headers = (table.areas & table_area_headers);
-            bool data = (table.areas & table_area_data);
-            bool totals = (table.areas & table_area_totals);
-
-            short count = 0;
-            if (headers)
-                ++count;
-            if (data)
-                ++count;
-            if (totals)
-                ++count;
-
-            bool multiple = count == 2;
-            if (multiple)
-                os << '[';
-
-            append_table_areas(os, table);
-
-            if (multiple)
-                os << ']';
-        }
-        else if (table.column_last == empty_string_id)
-        {
-            // single column.
-            os << '[';
-
-            bool multiple = false;
-            if (table.areas && table.areas != table_area_data)
-            {
-                if (append_table_areas(os, table))
-                {
-                    os << ',';
-                    multiple = true;
-                }
-            }
-
-            if (multiple)
-                os << '[';
-
-            append_name_string(os, mp_cxt, table.column_first);
-
-            if (multiple)
-                os << ']';
-
-            os << ']';
-        }
-        else
-        {
-            // column range.
-            os << '[';
-
-            if (table.areas && table.areas != table_area_data)
-            {
-                if (append_table_areas(os, table))
-                    os << ',';
-            }
-
-            os << '[';
-            append_name_string(os, mp_cxt, table.column_first);
-            os << "]:[";
-            append_name_string(os, mp_cxt, table.column_last);
-            os << "]]";
-        }
-
-        return os.str();
+        return to_string(mp_cxt, table);
     }
 
     virtual string get_column_name(col_t col) const
@@ -930,6 +1202,111 @@ private:
     const iface::formula_model_access* mp_cxt;
 };
 
+class excel_r1c1 : public formula_name_resolver
+{
+    const iface::formula_model_access* mp_cxt;
+
+    void write_sheet_name(ostringstream& os, const address_t& addr, const abs_address_t& pos) const
+    {
+        if (mp_cxt)
+        {
+            sheet_t sheet = addr.sheet;
+            if (!addr.abs_sheet)
+                sheet += pos.sheet;
+
+            append_sheet_name(os, *mp_cxt, sheet);
+            os << '!';
+        }
+    }
+
+public:
+    excel_r1c1(const iface::formula_model_access* cxt) : mp_cxt(cxt) {}
+
+    virtual formula_name_type resolve(const char* p, size_t n, const abs_address_t& pos) const
+    {
+        formula_name_type ret;
+        if (!n)
+            return ret;
+
+        if (resolve_function(p, n, ret))
+            return ret;
+
+        const char* p_last = p;
+        std::advance(p_last, n-1);
+
+        // Use the sheet where the cell is unless sheet name is explicitly given.
+        address_t parsed_addr(pos.sheet, 0, 0);
+
+        parse_address_result parse_res = parse_address_excel_r1c1(mp_cxt, p, p_last, parsed_addr);
+        switch (parse_res)
+        {
+            case parse_address_result::valid_address:
+            {
+                set_cell_reference(ret, parsed_addr);
+                return ret;
+            }
+            break;
+            case parse_address_result::range_expected:
+            {
+                ++p; // skip ':'
+                address_t parsed_addr2(0, 0, 0);
+                parse_address_result parse_res2 = parse_address_excel_r1c1(nullptr, p, p_last, parsed_addr2);
+                if (parse_res2 != parse_address_result::valid_address)
+                    return ret;
+
+                // For now, we assume the sheet index of the end address is identical
+                // to that of the begin address.
+                parsed_addr2.sheet = parsed_addr.sheet;
+
+                set_address(ret.range.first, parsed_addr);
+                set_address(ret.range.last, parsed_addr2);
+                ret.type = formula_name_type::range_reference;
+            }
+            break;
+            default:
+                ;
+        }
+
+        return ret;
+    }
+
+    virtual std::string get_name(const address_t& addr, const abs_address_t& pos, bool sheet_name) const
+    {
+        ostringstream os;
+
+        if (sheet_name)
+            write_sheet_name(os, addr, pos);
+
+        append_address_r1c1(os, addr, pos);
+        return os.str();
+    }
+
+    virtual std::string get_name(const range_t& range, const abs_address_t& pos, bool sheet_name) const
+    {
+        ostringstream os;
+
+        if (sheet_name)
+            write_sheet_name(os, range.first, pos);
+
+        append_address_r1c1(os, range.first, pos);
+        os << ':';
+        append_address_r1c1(os, range.last, pos);
+        return os.str();
+    }
+
+    virtual std::string get_name(const table_t& table) const
+    {
+        return to_string(mp_cxt, table);
+    }
+
+    virtual std::string get_column_name(col_t col) const
+    {
+        ostringstream os;
+        os << (col+1);
+        return os.str();
+    }
+};
+
 /**
  * Name resolver for ODFF formula expressions.
  */
@@ -1063,23 +1440,25 @@ private:
 
 }
 
-formula_name_resolver* formula_name_resolver::get(
+std::unique_ptr<formula_name_resolver> formula_name_resolver::get(
     formula_name_resolver_t type, const iface::formula_model_access* cxt)
 {
 
     switch (type)
     {
-        case formula_name_resolver_excel_a1:
-            return new excel_a1(cxt);
-        case formula_name_resolver_odff:
-            return new odff_resolver(cxt);
-        case formula_name_resolver_calc_a1:
-        case formula_name_resolver_excel_r1c1:
-        case formula_name_resolver_unknown:
+        case formula_name_resolver_t::excel_a1:
+            return std::unique_ptr<formula_name_resolver>(new excel_a1(cxt));
+        case formula_name_resolver_t::excel_r1c1:
+            return std::unique_ptr<formula_name_resolver>(new excel_r1c1(cxt));
+        case formula_name_resolver_t::odff:
+            return std::unique_ptr<formula_name_resolver>(new odff_resolver(cxt));
+        case formula_name_resolver_t::calc_a1:
+        case formula_name_resolver_t::unknown:
         default:
             ;
     }
-    return NULL;
+
+    return std::unique_ptr<formula_name_resolver>();
 }
 
 }
