@@ -19,8 +19,6 @@
 #include <cstring>
 #include <sstream>
 
-#include <boost/scoped_ptr.hpp>
-
 using namespace std;
 using namespace ixion;
 
@@ -57,6 +55,25 @@ void test_string_to_double()
     }
 }
 
+void test_string_pool()
+{
+    cout << "test string pool" << endl;
+    model_context cxt;
+
+    string_id_t s_table1 = cxt.append_string(IXION_ASCII("Table1"));
+    string_id_t s_table2 = cxt.append_string(IXION_ASCII("Table2"));
+    string_id_t s_cat = cxt.append_string(IXION_ASCII("Category"));
+    string_id_t s_val = cxt.append_string(IXION_ASCII("Value"));
+
+    cxt.dump_strings();
+
+    // Make sure these work correctly before proceeding further with the test.
+    assert(s_table1 == cxt.get_string_identifier(IXION_ASCII("Table1")));
+    assert(s_table2 == cxt.get_string_identifier(IXION_ASCII("Table2")));
+    assert(s_cat == cxt.get_string_identifier(IXION_ASCII("Category")));
+    assert(s_val == cxt.get_string_identifier(IXION_ASCII("Value")));
+}
+
 struct ref_name_entry
 {
     const char* name;
@@ -72,8 +89,7 @@ void test_name_resolver_excel_a1()
     cxt.append_sheet(IXION_ASCII("Two"), 1048576, 1024);
     cxt.append_sheet(IXION_ASCII("Three"), 1048576, 1024);
     cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 1024); // name with space
-    boost::scoped_ptr<formula_name_resolver> resolver(
-        formula_name_resolver::get(formula_name_resolver_excel_a1, &cxt));
+    auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
     assert(resolver);
 
     // Parse single cell addresses.
@@ -195,8 +211,13 @@ void test_name_resolver_table_excel_a1()
     string_id_t s_cat = cxt.append_string(IXION_ASCII("Category"));
     string_id_t s_val = cxt.append_string(IXION_ASCII("Value"));
 
-    boost::scoped_ptr<formula_name_resolver> resolver(
-        formula_name_resolver::get(formula_name_resolver_excel_a1, &cxt));
+    // Make sure these work correctly before proceeding further with the test.
+    assert(s_table1 == cxt.get_string_identifier(IXION_ASCII("Table1")));
+    assert(s_table2 == cxt.get_string_identifier(IXION_ASCII("Table2")));
+    assert(s_cat == cxt.get_string_identifier(IXION_ASCII("Category")));
+    assert(s_val == cxt.get_string_identifier(IXION_ASCII("Value")));
+
+    auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
     assert(resolver);
 
     struct {
@@ -258,6 +279,218 @@ void test_name_resolver_table_excel_a1()
     }
 }
 
+void test_name_resolver_excel_r1c1()
+{
+    cout << "test name resolver excel r1c1" << endl;
+    model_context cxt;
+    cxt.append_sheet(IXION_ASCII("One"), 1048576, 1024);
+    cxt.append_sheet(IXION_ASCII("Two"), 1048576, 1024);
+    cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 1024); // name with space
+    cxt.append_sheet(IXION_ASCII("80's Music"), 1048576, 1024);
+
+    auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_r1c1, &cxt);
+    assert(resolver);
+
+    // Parse single cell addresses for round-tripping.
+    ref_name_entry single_ref_names[] =
+    {
+        { "R2", false },
+        { "R[3]", false },
+        { "R[-10]", false },
+        { "C2", false },
+        { "C[3]", false },
+        { "C[-10]", false },
+        { "R1C1", false },
+        { "R[1]C2", false },
+        { "R2C[-2]", false },
+        { "R1C", false },
+        { "RC2", false },
+        { "One!R10C", true },
+        { "Two!C[-2]", true },
+        { "'A B C'!R100", true },
+        { 0, false }
+    };
+
+    for (size_t i = 0; single_ref_names[i].name; ++i)
+    {
+        const char* p = single_ref_names[i].name;
+        string name_r1c1(p);
+        formula_name_type res = resolver->resolve(name_r1c1.data(), name_r1c1.size(), abs_address_t());
+        if (res.type != formula_name_type::cell_reference)
+        {
+            cerr << "failed to resolve cell address: " << name_r1c1 << endl;
+            assert(false);
+        }
+
+        address_t addr = to_address(res.address);
+        string test_name = resolver->get_name(addr, abs_address_t(), single_ref_names[i].sheet_name);
+
+        if (name_r1c1 != test_name)
+        {
+            cerr << "failed to compile name from address: (name expected: "
+                << name_r1c1 << "; actual name created: " << test_name << ")" << endl;
+            assert(false);
+        }
+    }
+
+    // These are supposed to be all invalid.
+    const char* invalid_address[] = {
+        "F",
+        "RR",
+        "RC",
+        "R",
+        "C",
+        "R0C1",
+        "R[-2]C-1",
+        "R1C2:",
+        "R:",
+        "R2:",
+        "R[-3]:",
+        "C:",
+        "C3:",
+        "C[-4]:",
+        0
+    };
+
+    for (size_t i = 0; invalid_address[i]; ++i)
+    {
+        const char* p = invalid_address[i];
+        string name_r1c1(p);
+        formula_name_type res = resolver->resolve(name_r1c1.data(), name_r1c1.size(), abs_address_t());
+        if (res.type != formula_name_type::invalid)
+        {
+            cerr << "address " << name_r1c1 << " is expected to be invalid." << endl;
+            assert(false);
+        }
+    }
+
+    // These are supposed to be all valid.
+    const char* valid_address[] = {
+        "r1c2",
+        "r[-2]",
+        "c10",
+        0
+    };
+
+    for (size_t i = 0; valid_address[i]; ++i)
+    {
+        const char* p = valid_address[i];
+        string name_r1c1(p);
+        formula_name_type res = resolver->resolve(name_r1c1.data(), name_r1c1.size(), abs_address_t());
+        if (res.type != formula_name_type::cell_reference)
+        {
+            cerr << "address " << name_r1c1 << " is expected to be valid." << endl;
+            assert(false);
+        }
+    }
+
+    // Parse range addresses.
+    struct {
+        const char* name;
+        sheet_t sheet1;
+        row_t row1;
+        col_t col1;
+        sheet_t sheet2;
+        row_t row2;
+        col_t col2;
+        bool abs_sheet1;
+        bool abs_row1;
+        bool abs_col1;
+        bool abs_sheet2;
+        bool abs_row2;
+        bool abs_col2;
+    } range_tests[] = {
+        { "R1C1:R2C2", 0, 0, 0, 0, 1, 1, true, true, true, true, true, true },
+        { "R[-3]C[2]:R[1]C[4]", 0, -3, 2, 0, 1, 4, true, false, false, true, false, false },
+        { "R2:R4", 0, 1, column_unset, 0, 3, column_unset, true, true, false, true, true, false },
+        { "R[2]:R[4]", 0, 2, column_unset, 0, 4, column_unset, true, false, false, true, false, false },
+        { "C3:C6", 0, row_unset, 2, 0, row_unset, 5, true, false, true, true, false, true },
+        { "C[3]:C[6]", 0, row_unset, 3, 0, row_unset, 6, true, false, false, true, false, false },
+        { "Two!R2C2:R2C[100]", 1, 1, 1, 1, 1, 100, true, true, true, true, true, false },
+        { "'A B C'!R[2]:R[4]", 2, 2, column_unset, 2, 4, column_unset, true, false, false, true, false, false },
+        { 0, 0, 0, 0, 0, 0, 0, false, false, false, false, false, false }
+    };
+
+    for (size_t i = 0; range_tests[i].name; ++i)
+    {
+        string name_r1c1(range_tests[i].name);
+        cout << "Parsing " << name_r1c1 << endl;
+        formula_name_type res = resolver->resolve(&name_r1c1[0], name_r1c1.size(), abs_address_t());
+
+        assert(res.type == formula_name_type::range_reference);
+
+        assert(res.range.first.sheet == range_tests[i].sheet1);
+        assert(res.range.first.row == range_tests[i].row1);
+        assert(res.range.first.col == range_tests[i].col1);
+        assert(res.range.first.abs_sheet == range_tests[i].abs_sheet1);
+        if (res.range.first.row != row_unset)
+            // When row is unset, whether it's relative or absolute is not relevant.
+            assert(res.range.first.abs_row == range_tests[i].abs_row1);
+        if (res.range.first.col != column_unset)
+            // Same with unset column.
+            assert(res.range.first.abs_col == range_tests[i].abs_col1);
+
+        assert(res.range.last.sheet == range_tests[i].sheet2);
+        assert(res.range.last.row == range_tests[i].row2);
+        assert(res.range.last.col == range_tests[i].col2);
+        assert(res.range.last.abs_sheet == range_tests[i].abs_sheet2);
+        if (res.range.last.row != row_unset)
+            assert(res.range.last.abs_row == range_tests[i].abs_row2);
+        if (res.range.last.col != column_unset)
+            assert(res.range.last.abs_col == range_tests[i].abs_col2);
+    }
+
+    ref_name_entry range_ref_names[] =
+    {
+        { "R2C2:R3C3", false },
+        { "R[-3]C2:R[-1]C3", false },
+        { "'A B C'!R2:R4", true },
+        { "'80''s Music'!C[2]:C[4]", true },
+        { 0, false },
+    };
+
+    for (size_t i = 0; range_ref_names[i].name; ++i)
+    {
+        const char* p = range_ref_names[i].name;
+        string name_r1c1(p);
+        formula_name_type res = resolver->resolve(&name_r1c1[0], name_r1c1.size(), abs_address_t());
+        if (res.type != formula_name_type::range_reference)
+        {
+            cerr << "failed to resolve range address: " << name_r1c1 << endl;
+            assert(false);
+        }
+
+        range_t range = to_range(res.range);
+        string test_name = resolver->get_name(range, abs_address_t(), range_ref_names[i].sheet_name);
+
+        if (name_r1c1 != test_name)
+        {
+            cerr << "failed to compile name from range: (name expected: " << name_r1c1 << "; actual name created: " << test_name << ")" << endl;
+            assert(false);
+        }
+    }
+
+    struct {
+        col_t col;
+        string name;
+    } colnames[] = {
+        {   0,   "1" },
+        {   1,   "2" },
+        {  10,  "11" },
+        { 123, "124" },
+    };
+
+    for (size_t i = 0, n = IXION_N_ELEMENTS(colnames); i < n; ++i)
+    {
+        string colname = resolver->get_column_name(colnames[i].col);
+        if (colname != colnames[i].name)
+        {
+            cerr << "column name: expected='" << colnames[i].name << "', actual='" << colname << "'" << endl;
+            assert(false);
+        }
+    }
+}
+
 void test_name_resolver_odff()
 {
     cout << "test name resolver odff" << endl;
@@ -268,8 +501,7 @@ void test_name_resolver_odff()
     cxt.append_sheet(IXION_ASCII("A B C"), 1048576, 1024); // name with space
     cxt.append_sheet(IXION_ASCII("80's Music"), 1048576, 1024);
 
-    boost::scoped_ptr<formula_name_resolver> resolver(
-        formula_name_resolver::get(formula_name_resolver_odff, &cxt));
+    auto resolver = formula_name_resolver::get(formula_name_resolver_t::odff, &cxt);
     assert(resolver);
 
     // Parse single cell addresses.
@@ -398,8 +630,7 @@ void test_parse_and_print_expressions()
     cxt.append_string(IXION_ASCII("Table1"));
     cxt.append_string(IXION_ASCII("Category"));
     cxt.append_string(IXION_ASCII("Value"));
-    boost::scoped_ptr<formula_name_resolver> resolver(
-        formula_name_resolver::get(formula_name_resolver_excel_a1, &cxt));
+    auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
     assert(resolver);
 
     for (size_t i = 0; i < num_exps; ++i)
@@ -425,7 +656,7 @@ void test_function_name_resolution()
     };
 
     model_context cxt;
-    boost::scoped_ptr<formula_name_resolver> resolver(formula_name_resolver::get(ixion::formula_name_resolver_excel_a1, &cxt));
+    auto resolver = formula_name_resolver::get(ixion::formula_name_resolver_t::excel_a1, &cxt);
     size_t n = IXION_N_ELEMENTS(valid_names);
     for (size_t i = 0; i < n; ++i)
     {
@@ -461,8 +692,7 @@ void test_model_context_storage()
     cout << "test model context storage" << endl;
     {
         model_context cxt;
-        boost::scoped_ptr<formula_name_resolver> resolver(
-            formula_name_resolver::get(formula_name_resolver_excel_a1, &cxt));
+        auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
         assert(resolver);
 
         cxt.append_sheet(IXION_ASCII("test"), 1048576, 1024);
@@ -492,8 +722,7 @@ void test_model_context_storage()
 
     {
         model_context cxt;
-        boost::scoped_ptr<formula_name_resolver> resolver(
-            formula_name_resolver::get(formula_name_resolver_excel_a1, &cxt));
+        auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
         assert(resolver);
 
         cxt.append_sheet(IXION_ASCII("test"), 1048576, 1024);
@@ -549,8 +778,7 @@ void test_volatile_function()
     cout << "test volatile function" << endl;
 
     model_context cxt;
-    boost::scoped_ptr<formula_name_resolver> resolver(
-        formula_name_resolver::get(formula_name_resolver_excel_a1, &cxt));
+    auto resolver = formula_name_resolver::get(formula_name_resolver_t::excel_a1, &cxt);
     assert(resolver);
 
     cxt.append_sheet(IXION_ASCII("test"), 1048576, 1024);
@@ -625,8 +853,10 @@ int main()
 {
     test_size();
     test_string_to_double();
+    test_string_pool();
     test_name_resolver_excel_a1();
     test_name_resolver_table_excel_a1();
+    test_name_resolver_excel_r1c1();
     test_name_resolver_odff();
     test_address();
     test_parse_and_print_expressions();
