@@ -19,6 +19,10 @@
 
 #include <memory>
 #include <sstream>
+#include <unordered_map>
+#include <map>
+#include <vector>
+#include <iostream>
 
 #define DEBUG_MODEL_CONTEXT 0
 
@@ -65,7 +69,7 @@ bool set_shared_formula_tokens_to_cell(
     abs_address_t test = addr;
     test.row -= 1;
 
-    if (cxt.get_celltype(test) != celltype_formula)
+    if (cxt.get_celltype(test) != celltype_t::formula)
         // The neighboring cell is not a formula cell.
         return false;
 
@@ -133,9 +137,10 @@ bool set_shared_formula_tokens_to_cell(
 
 class model_context_impl : boost::noncopyable
 {
-    typedef boost::ptr_map<std::string, formula_cell> named_expressions_type;
-    typedef boost::ptr_vector<std::string> strings_type;
-    typedef boost::unordered_map<mem_str_buf, string_id_t, mem_str_buf::hash> string_map_type;
+    typedef std::map<std::string, unique_ptr<formula_cell>> named_expressions_type;
+    typedef std::vector<std::string> strings_type;
+    typedef std::vector<std::unique_ptr<std::string>> string_pool_type;
+    typedef std::unordered_map<mem_str_buf, string_id_t, mem_str_buf::hash> string_map_type;
     typedef std::deque<formula_tokens_t*> formula_tokens_store_type;
 
     typedef model_context::shared_tokens shared_tokens;
@@ -227,7 +232,10 @@ public:
     string_id_t add_string(const char* p, size_t n);
     const std::string* get_string(string_id_t identifier) const;
     size_t get_string_count() const;
+    void dump_strings() const;
+
     const column_store_t* get_column(sheet_t sheet, col_t col) const;
+    const column_stores_t* get_columns(sheet_t sheet) const;
 
     const formula_tokens_t* get_formula_tokens(sheet_t sheet, size_t identifier) const;
     size_t add_formula_tokens(sheet_t sheet, formula_tokens_t* p);
@@ -273,7 +281,7 @@ private:
     formula_tokens_store_type m_tokens;
     model_context::shared_tokens_type m_shared_tokens;
     strings_type m_sheet_names; ///< index to sheet name map.
-    strings_type m_strings;
+    string_pool_type m_strings;
     string_map_type m_string_map;
     string m_empty_string;
 };
@@ -281,19 +289,21 @@ private:
 void model_context_impl::set_named_expression(const char* p, size_t n, formula_cell* cell)
 {
     string name(p, n);
-    m_named_expressions.insert(name, cell);
+    m_named_expressions.insert(
+        named_expressions_type::value_type(
+            name, std::unique_ptr<formula_cell>(cell)));
 }
 
 formula_cell* model_context_impl::get_named_expression(const string& name)
 {
     named_expressions_type::iterator itr = m_named_expressions.find(name);
-    return itr == m_named_expressions.end() ? NULL : itr->second;
+    return itr == m_named_expressions.end() ? NULL : itr->second.get();
 }
 
 const formula_cell* model_context_impl::get_named_expression(const string& name) const
 {
     named_expressions_type::const_iterator itr = m_named_expressions.find(name);
-    return itr == m_named_expressions.end() ? NULL : itr->second;
+    return itr == m_named_expressions.end() ? NULL : itr->second.get();
 }
 
 const string* model_context_impl::get_named_expression_name(const formula_cell* expr) const
@@ -301,7 +311,7 @@ const string* model_context_impl::get_named_expression_name(const formula_cell*
     named_expressions_type::const_iterator itr = m_named_expressions.begin(), itr_end = m_named_expressions.end();
     for (; itr != itr_end; ++itr)
     {
-        if (itr->second == expr)
+        if (itr->second.get() == expr)
             return &itr->first;
     }
     return NULL;
@@ -349,7 +359,7 @@ sheet_t model_context_impl::append_sheet(
     // index of the new sheet.
     sheet_t sheet_index = m_sheets.size();
 
-    m_sheet_names.push_back(new string(p, n));
+    m_sheet_names.emplace_back(p, n);
     m_sheets.push_back(row_size, col_size);
     return sheet_index;
 }
@@ -361,10 +371,9 @@ string_id_t model_context_impl::append_string(const char* p, size_t n)
         return empty_string_id;
 
     string_id_t str_id = m_strings.size();
-    std::auto_ptr<string> ps(new string(p, n));
-    p = &(*ps)[0];
+    m_strings.push_back(make_unique<std::string>(p, n));
+    p = m_strings.back()->data();
     mem_str_buf key(p, n);
-    m_strings.push_back(ps);
     m_string_map.insert(string_map_type::value_type(key, str_id));
     return str_id;
 }
@@ -384,9 +393,9 @@ const std::string* model_context_impl::get_string(string_id_t identifier) const
         return &m_empty_string;
 
     if (identifier >= m_strings.size())
-        return NULL;
+        return nullptr;
 
-    return &m_strings[identifier];
+    return m_strings[identifier].get();
 }
 
 size_t model_context_impl::get_string_count() const
@@ -394,19 +403,51 @@ size_t model_context_impl::get_string_count() const
     return m_strings.size();
 }
 
+void model_context_impl::dump_strings() const
+{
+    {
+        cout << "string count: " << m_strings.size() << endl;
+        auto it = m_strings.begin(), ite = m_strings.end();
+        for (string_id_t sid = 0; it != ite; ++it, ++sid)
+        {
+            const std::string& s = **it;
+            cout << "* " << sid << ": '" << s << "' (" << (void*)s.data() << ")" << endl;
+        }
+    }
+
+    {
+        cout << "string map count: " << m_string_map.size() << endl;
+        auto it = m_string_map.begin(), ite = m_string_map.end();
+        for (; it != ite; ++it)
+        {
+            mem_str_buf key = it->first;
+            cout << "* key: '" << key << "' (" << (void*)key.get() << "; " << key.size() << "), value: " << it->second << endl;
+        }
+    }
+}
+
 const column_store_t* model_context_impl::get_column(sheet_t sheet, col_t col) const
 {
     if (static_cast<size_t>(sheet) >= m_sheets.size())
-        return NULL;
+        return nullptr;
 
     const worksheet& sh = m_sheets[sheet];
 
     if (static_cast<size_t>(col) >= sh.size())
-        return NULL;
+        return nullptr;
 
     return &sh[col];
 }
 
+const column_stores_t* model_context_impl::get_columns(sheet_t sheet) const
+{
+    if (static_cast<size_t>(sheet) >= m_sheets.size())
+        return nullptr;
+
+    const worksheet& sh = m_sheets[sheet];
+    return &sh.get_columns();
+}
+
 const formula_tokens_t* model_context_impl::get_formula_tokens(sheet_t sheet, size_t identifier) const
 {
     if (m_tokens.size() <= identifier)
@@ -879,18 +920,18 @@ celltype_t model_context_impl::get_celltype(const abs_address_t& addr) const
     switch (gmcell_type)
     {
         case mdds::mtv::element_type_empty:
-            return celltype_empty;
+            return celltype_t::empty;
         case mdds::mtv::element_type_numeric:
-            return celltype_numeric;
+            return celltype_t::numeric;
         case mdds::mtv::element_type_ulong:
-            return celltype_string;
+            return celltype_t::string;
         case element_type_formula:
-            return celltype_formula;
+            return celltype_t::formula;
         default:
             throw general_error("unknown cell type");
     }
 
-    return celltype_unknown;
+    return celltype_t::unknown;
 }
 
 double model_context_impl::get_numeric_value(const abs_address_t& addr) const
@@ -1269,11 +1310,21 @@ size_t model_context::get_string_count() const
     return mp_impl->get_string_count();
 }
 
+void model_context::dump_strings() const
+{
+    mp_impl->dump_strings();
+}
+
 const column_store_t* model_context::get_column(sheet_t sheet, col_t col) const
 {
     return mp_impl->get_column(sheet, col);
 }
 
+const column_stores_t* model_context::get_columns(sheet_t sheet) const
+{
+    return mp_impl->get_columns(sheet);
+}
+
 void model_context::get_all_formula_cells(dirty_formula_cells_t& cells) const
 {
     mp_impl->get_all_formula_cells(cells);
