@@ -86,20 +86,11 @@ public:
 
 // ============================================================================
 
-model_parser::parse_error::parse_error(const string& msg)
+model_parser::parse_error::parse_error(const string& msg) : general_error()
 {
     ostringstream os;
     os << "parse error: " << msg;
-    m_msg = os.str();
-}
-
-model_parser::parse_error::~parse_error() throw()
-{
-}
-
-const char* model_parser::parse_error::what() const throw()
-{
-    return m_msg.c_str();
+    set_message(os.str());
 }
 
 // ============================================================================
@@ -113,8 +104,8 @@ model_parser::model_parser(const string& filepath, size_t thread_count) :
     m_context(),
     m_session_handler(m_context),
     m_table_handler(),
-    mp_table_entry(NULL),
-    mp_name_resolver(formula_name_resolver::get(formula_name_resolver_excel_a1, &m_context)),
+    mp_table_entry(nullptr),
+    mp_name_resolver(formula_name_resolver::get(formula_name_resolver_t::excel_a1, &m_context)),
     m_filepath(filepath),
     m_thread_count(thread_count),
     m_print_separator(true)
@@ -488,7 +479,8 @@ void model_parser::push_table()
     cout << endl;
 
     cout << "totals row count: " << mp_table_entry->totals_row_count << endl;
-    m_table_handler.insert(mp_table_entry.release());
+    m_table_handler.insert(mp_table_entry);
+    assert(!mp_table_entry);
 }
 
 void model_parser::parse_table_columns(const mem_str_buf& str)
@@ -553,7 +545,7 @@ void model_parser::check()
 
                 switch (m_context.get_celltype(addr))
                 {
-                    case celltype_formula:
+                    case celltype_t::formula:
                     {
                         const formula_cell* fcell = m_context.get_formula_cell(addr);
                         const formula_result* res_cell = fcell->get_result_cache();
@@ -568,7 +560,7 @@ void model_parser::check()
                         }
                     }
                     break;
-                    case celltype_numeric:
+                    case celltype_t::numeric:
                     {
                         double actual_val = m_context.get_numeric_value(addr);
                         if (actual_val != res.get_value())
@@ -579,7 +571,7 @@ void model_parser::check()
                         }
                     }
                     break;
-                    case celltype_string:
+                    case celltype_t::string:
                     {
                         string_id_t str_id = m_context.get_string_identifier(addr);
 
