@@ -12,7 +12,6 @@
 #include "ixion/model_context.hpp"
 
 #include <string>
-#include <boost/scoped_ptr.hpp>
 
 namespace ixion {
 
@@ -37,7 +36,7 @@ public:
 
 private:
     const model_context& m_context;
-    boost::scoped_ptr<formula_name_resolver> mp_resolver;
+    std::unique_ptr<formula_name_resolver> mp_resolver;
     std::string m_cell_name;
 };
 
